' //////////////////////////////////////////////////////////////////////
' TITLE: EPM_HDMF_LITE_DEMO_011.SPIN 
'
' DESCRIPTION:
'   Demonstrates the features of the Hydra Dense Music Format (HDMF) player.        
'
' AUTHOR:  Eric Moyer
'
' UPDATES: Search the Parallax forums at http://forums.parallax.com for the latest version.
'
' CONTROLS:
'          Two NES gamepads, or a gamepad and the keyboard
'          Press START or <Spacebar> at the GAME OVER screen to return to the options menu.
'
'          Gamepad 0    Function 
'          ---------    ------------------------------------------------------------------------
'          UP           Move cursor up   
'          DOWN         Move cursor down
'          A            Start song
'          B            Stop song
'
' //////////////////////////////////////////////////////////////////////
'
'  Revision History
'
'  Rev  Date      Description
'  ---  --------  ---------------------------------------------------
'  010  05-22-07  Initial Creation
'
' //////////////////////////////////////////////////////////////////////


'///////////////////////////////////////////////////////////////////////
' CONSTANTS SECTION ////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////

CON

  _clkmode = xtal2 + pll8x            ' enable external clock and pll times 8
  _xinfreq = 10_000_000 + 0000        ' set frequency to 10 MHZ plus some error for xtals that are not exact add 1000-5000 usually works
  _stack = ($1800 + $1800 + 64) >> 2  ' accomodate display memory and stack

  'DEBUG OPTIONS:
  DEBUG__ENABLE_MUSIC_DEBUG     = false
  DEUBG__SHOW_SYSTEM_COUNTER    = false
                           
  MAX_SONGS = 10
  
  ' graphics driver and screen constants
  PARAMCOUNT        = 14        
  OFFSCREEN_BUFFER  = $5000           ' offscreen buffer
  ONSCREEN_BUFFER   = $6800           ' onscreen buffer

  ' size of graphics tile map
  X_TILES           = 16
  Y_TILES           = 12
  
  SCREEN_WIDTH      = 256
  SCREEN_HEIGHT     = 192 

  ' Display constants 
  TEXT_LINE_SPACING = 13

  ' angular constants to make object declarations easier
  ANG_0    = $0000
  ANG_360  = $2000
  ANG_240  = ($2000*2/3)
  ANG_180  = ($2000/2)
  ANG_270  = ($2000*3/4)
  ANG_120  = ($2000/3)
  ANG_90   = ($2000/4)
  ANG_60   = ($2000/6)
  ANG_45   = ($2000/8)
  ANG_30   = ($2000/12)
  ANG_22_5 = ($2000/16)
  ANG_15   = ($2000/24)
  ANG_10   = ($2000/36)
  ANG_5    = ($2000/72)
  ANG_1    = ($2000/360)

  ' constants for math functions
  SIN      = 0
  COS      = 1

  ' NES bit encodings
  NES_RIGHT  = %00000001
  NES_LEFT   = %00000010
  NES_DOWN   = %00000100
  NES_UP     = %00001000
  NES_START  = %00010000
  NES_SELECT = %00100000
  NES_B      = %01000000
  NES_A      = %10000000

  'Menu Flags
  MENU_FLAG__A_HELD                 = %00000001
  MENU_FLAG__UP_DOWN_HELD           = %00000010
  MENU_FLAG__REPEAT_GO_TIME_MET     = %00000100
  MENU_FLAG__DO_REPEAT_CLICK        = %00001000
  MENU_FLAG__START_HELD             = %00010000


  'Vector drawing commands
  VECTOR_MOVE           = $4000
  VECTOR_DRAW           = $8000
  VECTOR_END            = $0000

  NUM_DEBUG_LONGS       = 8

  'Loop sim FPS rate control
  LOOP_SIM_FPS_MAX = 1000
  LOOP_SIM_FPS_MIN = 0005
  LOOP_SIM_FPS_GAUGE_X = -40
  LOOP_SIM_FPS_GAUGE_Y = -55

'///////////////////////////////////////////////////////////////////////
' VARIABLES SECTION ////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////

VAR
    
  long  my_stack[30]
  
  long  tv_status     '0/1/2 = off/visible/invisible           read-only
  long  tv_enable     '0/? = off/on                            write-only
  long  tv_pins       '%ppmmm = pins                           write-only
  long  tv_mode       '%ccinp = chroma,interlace,ntsc/pal,swap write-only
  long  tv_screen     'pointer to screen (words)               write-only
  long  tv_colors     'pointer to colors (longs)               write-only               
  long  tv_hc         'horizontal cells                        write-only
  long  tv_vc         'vertical cells                          write-only
  long  tv_hx         'horizontal cell expansion               write-only
  long  tv_vx         'vertical cell expansion                 write-only
  long  tv_ho         'horizontal offset                       write-only
  long  tv_vo         'vertical offset                         write-only
  long  tv_broadcast  'broadcast frequency (Hz)                write-only
  long  tv_auralcog   'aural fm cog                            write-only

  word  screen[x_tiles * y_tiles] ' storage for screen tile map
  long  colors[64]                ' color look up table

  ' songs
  long song[MAX_SONGS]

  ' nes gamepad vars
  long nes_buttons
  long button_temp

  long debug_data[NUM_DEBUG_LONGS]

  ' Game loop simulator control
  byte LOOP_SIM__start_song        'Boolean.  Set to tell loop sim to start a song.
  byte LOOP_SIM__stop_song         'Boolean.  Set to tell loop sim to stop a song.
  byte LOOP_SIM__song_is_playing   'Boolean.  Returned by loop sim.  Set if song playing. 
  long LOOP_SIM__song_address      'Song address to play on a "start song".
  long LOOP_SIM__song_flags        'Song flags to use on a "start song".
  long LOOP_SIM__frame_dealy       'Delay in cnt ticks to simulate game loop frame rate.
  long LOOP_SIM__loop_time         'Actual game loop frame rate in cnt ticks (reported).

  long FPS_percentage
  
'///////////////////////////////////////////////////////////////////////
' OBJECT DECLARATION SECTION ///////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////
OBJ

  tv        : "EPM_tv_drv_011.spin"                ' Paralax tv driver, modified for 2 color 
  gr        : "EPM_graphics_drv_spacewar_012.spin" ' Paralax graphics driver, modified for 2 color
  hdmf_lite : "EPM_HDMF_Lite_driver_010.spin"      ' HDMF Lite driver   

'///////////////////////////////////////////////////////////////////////
' EXPORT PUBLICS  //////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////

PUB start | i , dx, dy

  '//// GLOBAL INITIALIZTION
  
  'start tv
  longmove(@tv_status, @tvparams, paramcount)
  tv_screen := @screen
  tv_colors := @colors
  tv.start(@tv_status)

  'init debug LED
  DIRA [0] := 1 ' output 
  
  'init colors
  repeat i from 0 to 64
    colors[i] := $00001010 * (i+4) & $F + $FB060C02

  'init tile screen
  repeat dx from 0 to tv_hc - 1
    repeat dy from 0 to tv_vc - 1
      screen[dy * tv_hc + dx] := onscreen_buffer >> 6 + dy + dx * tv_vc + ((dy & $3F) << 10)

  'start and setup graphics
  gr.start
  gr.setup(16, 12, 128, 96, offscreen_buffer)

  'Run main menu handler
  Menu_Engine 

PUB Menu_Engine | i, y, song_size, selected_option, string_ptr, num_options, menu_flags, key_repeat_ticks, current_option_mask, done, fps, loop_song
 
  selected_option := 0
  menu_flags      := 0
  done            := false
  FPS_percentage  := 100
  loop_song       := false
  LOOP_SIM__frame_dealy := 100000 'temporary frame delay until the real one gets generated below

  'LOOP_SIM__frame_dealy := 80_000_000 / (CONSTANT(LOOP_SIM_FPS_MAX - LOOP_SIM_FPS_MIN) / (101-FPS_percentage))

  repeat i from 0 to MAX_SONGS -1
    song[i] := 0

  i:=0
  song[i++] := @_SONG_ms_pacman
  song[i++] := @_SONG_tron_failure
  song[i++] := @_SONG_dk_background
  song[i++] := @_SONG_dk_start
  song[i++] := @_SONG_dk_hammer
  song[i++] := @_SONG_dk_how_high
  song[i++] := @_SONG_end  

  'Initialize the game loop simulator
  LOOP_SIM__start_song  := false
  LOOP_SIM__stop_song   := false

  'Start the game loop simulator
  cognew(Game_loop_simulator, @my_stack)
    
  repeat while not done

    '//// Clear screen
    gr.clear        ' Clear bitmap

    '//// GET USER INPUT
    nes_buttons := NES_Read_Gamepad
    nes_buttons &= $ff
    if nes_buttons == $ff
      nes_buttons := 0
    
    '//// Draw titles
    gr.textmode(1,1,6,1)
    gr.colorwidth(2,0)

    y:=86
    gr.text(-43, y, string("HDMF LITE DEMO"))
     
    y -= TEXT_LINE_SPACING
    gr.text(-34, y, string("VERSION 1.1"))

    y:=-32
    if LOOP_SIM__song_is_playing 
      gr.text(-100, y, string("PLAYING"))
    else
      gr.text(-100, y, string("IDLE"))
     
    y-=TEXT_LINE_SPACING  
    if loop_song
      gr.text(-100, y, string("LOOPING:ON  ('START' TO TOGGLE)"))
    else
      gr.text(-100, y, string("LOOPING:OFF ('START' TO TOGGLE)"))  
      
    y-=TEXT_LINE_SPACING
    fps := 80_000_000 / LOOP_SIM__loop_time             
    Int_To_String(@uint_string_0, fps)
    gr.text(-100, y, string("FPS:"))
    gr.text(-75, y, @uint_string_0) 
     
    if !DEBUG__ENABLE_MUSIC_DEBUG
      y:=-79
      gr.text(-110, y, string("WRITTEN BY ERIC MOYER. FOR THE LATEST"))
      y -= TEXT_LINE_SPACING 
      gr.text(-123, y, string("VERSION SEARCH HTTP://FORUMS.PARALLAX.COM"))
    else
      y := 20
      repeat i from 0 to NUM_DEBUG_LONGS - 1 
        Int_To_Hex_String(@hex_string_0, debug_data[i])
        gr.text(-120,y,@hex_string_0)
        gr.finish
        y-= 13

    if DEUBG__SHOW_SYSTEM_COUNTER
       Int_To_HEX_String(@hex_string_0, cnt)
       gr.text(-120,86,@hex_string_0)
      gr.finish
       
    '//// Draw FPS control gauge
    gr.plot(LOOP_SIM_FPS_GAUGE_X, LOOP_SIM_FPS_GAUGE_Y)
    gr.line(CONSTANT(LOOP_SIM_FPS_GAUGE_X+103), LOOP_SIM_FPS_GAUGE_Y)
    gr.line(CONSTANT(LOOP_SIM_FPS_GAUGE_X+103), LOOP_SIM_FPS_GAUGE_Y-5)
    gr.line(LOOP_SIM_FPS_GAUGE_X, LOOP_SIM_FPS_GAUGE_Y-5)
    gr.line(LOOP_SIM_FPS_GAUGE_X, LOOP_SIM_FPS_GAUGE_Y)
    
    gr.plot(CONSTANT(LOOP_SIM_FPS_GAUGE_X+2), CONSTANT(LOOP_SIM_FPS_GAUGE_Y-2))
    gr.line(CONSTANT(LOOP_SIM_FPS_GAUGE_X+2)+FPS_percentage-1, CONSTANT(LOOP_SIM_FPS_GAUGE_Y-2))
    gr.plot(CONSTANT(LOOP_SIM_FPS_GAUGE_X+2), CONSTANT(LOOP_SIM_FPS_GAUGE_Y-3))
    gr.line(CONSTANT(LOOP_SIM_FPS_GAUGE_X+2)+FPS_percentage-1, CONSTANT(LOOP_SIM_FPS_GAUGE_Y-3))

    '//// Set FPS delay for game loop simulation
    LOOP_SIM__frame_dealy := 80_000_000 / (CONSTANT(LOOP_SIM_FPS_MAX - LOOP_SIM_FPS_MIN) / (101-FPS_percentage))         

    '//// Draw options states
    string_ptr := @option_strings
    y := 55
    num_options := 0
    current_option_mask := $00000001
    repeat while (byte[string_ptr] <> $ff)
     
      '//// Draw option name
      gr.text(-60, y, string_ptr)
      repeat while byte[string_ptr] <> $0
          ++string_ptr
      ++string_ptr
      
      '//// Draw song size
      song_size := song[num_options+1]  - song[num_options]
      Int_To_String(@uint_string_0, song_size )
      gr.text(-100, y , @uint_string_0)
      
      '//// Play song
      if (nes_buttons & NES_A) and (selected_option == num_options) and not(menu_flags & MENU_FLAG__A_HELD)
        LOOP_SIM__song_address := song[num_options]
        if loop_song
          LOOP_SIM__song_flags   := hdmf_lite#HDMF__FLAG_LOOP_SONG
        else
          LOOP_SIM__song_flags   := 0
        LOOP_SIM__start_song   := true  
       
      '//// Advance to next line
      y -= 13
      ++num_options
      current_option_mask <<= 1
     
    '//// Point to current option
    y :=55 -(13 * selected_option)
    gr.vec(-120, y, $0020, ANG_0, @player_ship0)
     
    '//// PROCESS MENU SELECTION, ON KEY DOWN EVENTS WITH KEY REPEATS
    if menu_flags & MENU_FLAG__UP_DOWN_HELD 
      ++key_repeat_ticks
      if not(nes_buttons & CONSTANT(NES_DOWN | NES_UP))
        menu_flags &= CONSTANT(!(MENU_FLAG__UP_DOWN_HELD | MENU_FLAG__REPEAT_GO_TIME_MET | MENU_FLAG__DO_REPEAT_CLICK))
      if not(menu_flags & MENU_FLAG__REPEAT_GO_TIME_MET) and (key_repeat_ticks => 8)
        menu_flags |= CONSTANT(MENU_FLAG__REPEAT_GO_TIME_MET | MENU_FLAG__DO_REPEAT_CLICK)
      if (menu_flags & MENU_FLAG__REPEAT_GO_TIME_MET) and (key_repeat_ticks => 2)
        menu_flags |= MENU_FLAG__DO_REPEAT_CLICK  
        
    if (not(menu_flags & MENU_FLAG__UP_DOWN_HELD)) or (menu_flags & MENU_FLAG__DO_REPEAT_CLICK)
      if nes_buttons & NES_UP  
        menu_flags |= MENU_FLAG__UP_DOWN_HELD
        if selected_option == 0
          selected_option := num_options - 1
        else
          --selected_option
      if nes_buttons & NES_DOWN  
        menu_flags |= MENU_FLAG__UP_DOWN_HELD  
        if selected_option == num_options - 1
          selected_option := 0
        else
          ++selected_option
      key_repeat_ticks := 0
      menu_flags &= !MENU_FLAG__DO_REPEAT_CLICK

    '//// PROCESS "STOP SONG"
    if (nes_buttons & NES_B)
       LOOP_SIM__stop_song := true

    '//// PROCESS FPS PERCENTAGE MODIFICATION
    if (nes_buttons & NES_LEFT)
       --FPS_percentage
       if FPS_percentage < 1
         FPS_percentage := 1

    if (nes_buttons & NES_RIGHT)
       ++FPS_percentage
       if FPS_percentage > 100
         FPS_percentage := 100

    '//// PROCESS LOOPING TOGGLE
    if (nes_buttons & NES_START) and not (menu_flags & MENU_FLAG__START_HELD)
      loop_song := not loop_song

    '//// PROCESS BUTTON HELD FLAGS
    if (nes_buttons & NES_A) 
       menu_flags |= MENU_FLAG__A_HELD
    else 
       menu_flags &= !MENU_FLAG__A_HELD

    if (nes_buttons & NES_START) 
       menu_flags |= MENU_FLAG__START_HELD
    else 
       menu_flags &= !MENU_FLAG__START_HELD
       

    '//// COPY BITMAP TO DISPLAY
    gr.copy(onscreen_buffer)

PUB Game_loop_simulator | loop_start_cnt, loop_end_cnt
''
'' Simualtes a game execution loop for HDMF Lite testing
''
  'LOOP_SIM__loop_time := 8_000_000
  'repeat
  
  'Start the hdmf lite driver
  if( DEBUG__ENABLE_MUSIC_DEBUG )
    hdmf_lite.start(@debug_data)
  else
    hdmf_lite.start(0)

  'Simulate a game loop
  repeat
    'Record main game loop start time
    loop_start_cnt := cnt
    
    if LOOP_SIM__start_song
      'Start a song
      hdmf_lite.play_song(LOOP_SIM__song_address, LOOP_SIM__song_flags)
      'Clear the start flag
      LOOP_SIM__start_song := false
      
    elseif LOOP_SIM__stop_song
      'Stop a song
      hdmf_lite.stop_song
      'Clear the stop flag
      LOOP_SIM__stop_song  := false

    'Check song status
    if hdmf_lite.song_is_playing
       LOOP_SIM__song_is_playing  := true
    else
       LOOP_SIM__song_is_playing  := false

    'Simulate main game logic with a delay
    waitcnt(LOOP_SIM__frame_dealy + cnt)
    'waitcnt(100000 + cnt)               

    'Process song playback
    hdmf_lite.do_playback

    'Record main game loop end time
    loop_end_cnt := cnt

    'If the counter did not loop
    if not ((loop_start_cnt & $80000000) and not(loop_end_cnt & $80000000))
      LOOP_SIM__loop_time := loop_end_cnt - loop_start_cnt


PUB Int_To_Hex_String(str, i) | t, nibble
   
  ' does an sprintf(str, "%08lX", i); job
  ' assumes that '0x' is already in the first 2 character positions of the string
  str+=9
  repeat t from 0 to 7
    nibble := i & $0000000f
    if nibble =< 9
      BYTE[str] := 48 {'0'} + nibble
    else
      BYTE[str] := CONSTANT(65 {'A'} - 10) + nibble
    str--
    i >>= 4

PUB Int_To_String(str, i) | t
   
  ' does an sprintf(str, "%05d", i); job
  str+=4
  repeat t from 0 to 4
    BYTE[str] := 48+(i // 10)
    i/=10
    str--

PUB WaitForStartButton
  nes_buttons := 0
  'hang here until 'START' or '<SPACEBAR>' is pressed    
  repeat while not (nes_buttons & CONSTANT( NES_START | (NES_START << 8) ))
    '//// Read NES controller buttons
    nes_buttons := NES_Read_Gamepad

  'hang here until 'START' and '<SPACEBAR>' are released   
  repeat while (nes_buttons & CONSTANT( NES_START | (NES_START << 8) )) 
    '//// Read NES controller buttons
    nes_buttons := NES_Read_Gamepad

PUB NES_Read_Gamepad : nes_bits        |  i

' //////////////////////////////////////////////////////////////////
' NES Game Paddle Read
' //////////////////////////////////////////////////////////////////       
' reads both gamepads in parallel encodes 8-bits for each in format
' right game pad #1 [15..8] : left game pad #0 [7..0]
'
' set I/O ports to proper direction
' P3 = JOY_CLK      (4)
' P4 = JOY_SH/LDn   (5) 
' P5 = JOY_DATAOUT0 (6)
' P6 = JOY_DATAOUT1 (7)
' NES Bit Encoding
'
' RIGHT  = %00000001
' LEFT   = %00000010
' DOWN   = %00000100
' UP     = %00001000
' START  = %00010000
' SELECT = %00100000
' B      = %01000000
' A      = %10000000

' step 1: set I/Os
DIRA [3] := 1 ' output
DIRA [4] := 1 ' output
DIRA [5] := 0 ' input
DIRA [6] := 0 ' input

' step 2: set clock and latch to 0
OUTA [3] := 0 ' JOY_CLK = 0
OUTA [4] := 0 ' JOY_SH/LDn = 0
'Delay(1)

' step 3: set latch to 1
OUTA [4] := 1 ' JOY_SH/LDn = 1
'Delay(1)

' step 4: set latch to 0
OUTA [4] := 0 ' JOY_SH/LDn = 0

' step 5: read first bit of each game pad

' data is now ready to shift out
' first bit is ready 
nes_bits := 0

' left controller
nes_bits := INA[5] | (INA[6] << 8)

' step 7: read next 7 bits
repeat i from 0 to 6
 OUTA [3] := 1 ' JOY_CLK = 1
 'Delay(1)             
 OUTA [3] := 0 ' JOY_CLK = 0
 nes_bits := (nes_bits << 1)
 nes_bits := nes_bits | INA[5] | (INA[6] << 8)

 'Delay(1)             
' invert bits to make positive logic
nes_bits := (!nes_bits & $FFFF)

' //////////////////////////////////////////////////////////////////



'///////////////////////////////////////////////////////////////////////
' DATA SECTION /////////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////

DAT

' TV PARAMETERS FOR DRIVER /////////////////////////////////////////////

tvparams                long    0               'status
                        long    1               'enable
                        long    %011_0000       'pins
                        long    %0000           'mode
                        long    0               'screen
                        long    0               'colors
                        long    x_tiles         'hc
                        long    y_tiles         'vc
                        long    10              'hx timing stretch
                        long    1               'vx
                        long    0               'ho
                        long    0               'vo
                        long    55_250_000      'broadcast
                        long    0               'auralcog

' POLYGON OBJECTS ///////////////////////////////////////////////////////
  
player_ship0            word    VECTOR_MOVE + ANG_0             
                        word    50

                        word    VECTOR_DRAW + ANG_120+ANG_10
                        word    50

                        word    VECTOR_DRAW + ANG_180
                        word    10

                        word    VECTOR_DRAW + ANG_240-ANG_10  
                        word    50

                        word    VECTOR_DRAW + ANG_0
                        word    50

                        word    VECTOR_END
                      

fixed_pt_string_0        byte    "-00000.0000",0    'text
int_string_0             byte    "-00000",0         'text
uint_string_0            byte    "00000",0          'text 
hex_string_0             byte    "0X00000000",0     'text

option_strings           byte    "MS. PACMAN",0    
                         byte    "TRON (FAILURE)",0
                         byte    "DONKEY KONG - BACKGROUND",0  
                         byte    "DONKEY KONG - START",0
                         byte    "DONKEY KONG - HAMMER",0
                         byte    "DONKEY KONG - HOW HIGH?",0
                         byte    $ff   'end of option string list

_SONG_ms_pacman
        byte    $02, $EF, $AD, $79, $35          'Instrument 0
        byte    $03, $EF, $AD, $79, $35          'Instrument 1
        byte    $FF                              'End of Instrument list
        byte    $00, $02, $0B, $2C, $08
        byte    $08, $02, $4B, $2C, $08
        byte    $09, $02, $93, $2C, $08
        byte    $08, $00, $AF, $0C, $18
        byte    $00, $22, $BA, $2C, $18
        byte    $19, $03, $70, $2C, $18
        byte    $19, $00, $83, $0C, $18
        byte    $00, $23, $10, $2C, $18
        byte    $19, $03, $A4, $2C, $18
        byte    $19, $00, $AF, $0C, $18
        byte    $00, $23, $70, $2C, $0C
        byte    $0C, $23, $A4, $2C, $0C
        byte    $0D, $04, $17, $2C, $0C
        byte    $0D, $03, $70, $2C, $0C
        byte    $0C, $00, $C4, $0C, $0C
        byte    $00, $23, $10, $2C, $18
        byte    $0C, $00, $DC, $0C, $0C
        byte    $0D, $03, $A4, $2C, $18
        byte    $00, $20, $E9, $0C, $0C
        byte    $0D, $20, $C4, $0C, $0C
        byte    $0C, $00, $DC, $0C, $0C
        byte    $00, $23, $70, $2C, $0C
        byte    $0C, $00, $C4, $0C, $0C
        byte    $00, $23, $A4, $2C, $0C
        byte    $0D, $04, $17, $2C, $0C
        byte    $00, $20, $AF, $0C, $0C
        byte    $0D, $03, $70, $2C, $0C
        byte    $00, $20, $DC, $0C, $0C
        byte    $0C, $00, $C4, $0C, $0C
        byte    $00, $23, $A4, $2C, $0C
        byte    $0C, $00, $AF, $0C, $0C
        byte    $00, $24, $17, $2C, $0C
        byte    $0D, $00, $A5, $0C, $0C
        byte    $00, $24, $97, $2C, $0C
        byte    $0D, $00, $C4, $0C, $0C
        byte    $00, $25, $27, $2C, $0C
        byte    $0C, $05, $75, $2C, $18
        byte    $00, $20, $AF, $0C, $18
        byte    $19, $00, $83, $0C, $18
        byte    $00, $25, $27, $2C, $18
        byte    $19, $00, $AF, $0C, $18
        byte    $00, $25, $75, $2C, $18
        byte    $ff

_SONG_tron_failure
        byte    $02, $EF, $AD, $79, $35          'Instrument 0
        byte    $FF                              'End of Instrument list
        byte    $00, $01, $EE, $0F, $0D
        byte    $00, $23, $DC, $0F, $2B
        byte    $00, $40, $7B, $0F, $56
        byte    $00, $61, $5D, $0F, $0D
        byte    $0D, $01, $EE, $0F, $0D
        byte    $00, $61, $5D, $0F, $0D
        byte    $0E, $01, $5D, $0F, $0D
        byte    $00, $61, $EE, $0F, $0D
        byte    $10, $01, $EE, $0F, $0D
        byte    $00, $25, $27, $0F, $20
        byte    $00, $61, $5D, $0F, $0D
        byte    $0D, $01, $EE, $0F, $0D
        byte    $00, $61, $5D, $0F, $0D
        byte    $0E, $01, $5D, $0F, $0D
        byte    $00, $61, $EE, $0F, $0D
        byte    $05, $23, $DC, $0F, $0B
        byte    $0B, $04, $DD, $0F, $20
        byte    $00, $20, $A5, $0F, $40
        byte    $00, $41, $4A, $0F, $0D
        byte    $01, $61, $D2, $0F, $0D
        byte    $0C, $41, $4A, $0F, $0D
        byte    $02, $61, $D2, $0F, $0D
        byte    $0B, $41, $4A, $0F, $0D
        byte    $02, $61, $D2, $0F, $0D
        byte    $04, $03, $A4, $0F, $15
        byte    $0B, $41, $37, $0F, $0D
        byte    $00, $61, $EE, $0F, $0D
        byte    $0D, $01, $EE, $0F, $0D
        byte    $00, $44, $97, $0F, $0D
        byte    $00, $61, $37, $0F, $0D
        byte    $08, $20, $9C, $0F, $15
        byte    $05, $01, $9F, $0F, $0D
        byte    $00, $41, $26, $0F, $0D
        byte    $00, $63, $70, $0F, $0D
        byte    $0E, $04, $55, $0F, $0D
        byte    $02, $20, $93, $0F, $56
        byte    $00, $41, $9F, $0F, $0D
        byte    $00, $61, $26, $0F, $0D
        byte    $0E, $01, $26, $0F, $0D
        byte    $00, $44, $17, $0F, $0D
        byte    $00, $61, $9F, $0F, $0D
        byte    $0D, $03, $A4, $0F, $0D
        byte    $00, $41, $26, $0F, $0D
        byte    $00, $61, $9F, $0F, $0D
        byte    $0E, $03, $3F, $0F, $1B
        byte    $02, $41, $9F, $0F, $0D
        byte    $00, $61, $26, $0F, $0D
        byte    $0E, $41, $9F, $0F, $0D
        byte    $00, $61, $26, $0F, $0D
        byte    $0D, $01, $9F, $0F, $0D
        byte    $00, $42, $E4, $0F, $1B
        byte    $00, $61, $26, $0F, $0D
        byte    $10, $01, $72, $0F, $2B
        byte    $00, $20, $AF, $0F, $0B
        byte    $00, $60, $93, $0F, $2B
        byte    $0E, $21, $06, $0F, $20
        byte    $00, $44, $17, $0F, $0D
        byte    $0D, $43, $DC, $0F, $0D
        byte    $0D, $43, $70, $0F, $0D
        byte    $03, $00, $7B, $0F, $2B
        byte    $00, $60, $AF, $0F, $1B
        byte    $00, $81, $5D, $0F, $15
        byte    $0D, $23, $10, $0F, $0D
        byte    $08, $41, $72, $0F, $15
        byte    $06, $22, $BA, $0F, $0D
        byte    $00, $60, $F7, $0F, $0D
        byte    $0D, $20, $E9, $0F, $56
        byte    $03, $00, $75, $0F, $56
        byte    $00, $42, $93, $0F, $56
        byte    $00, $61, $4A, $0F, $56
        byte    $ff

_SONG_dk_background
        byte    $03, $EF, $AD, $79, $35          'Instrument 0
        byte    $FF                              'End of Instrument list
        byte    $00, $00, $83, $0F, $14
        byte    $3C, $20, $A5, $0F, $14
        byte    $28, $40, $C4, $0F, $14
        byte    $14, $60, $DC, $0F, $14
        byte    $14, $80, $C4, $0F, $14
        byte    $14, $80, $01, $01, $01
        byte    $ff

_SONG_dk_start
        byte    $03, $EF, $AD, $79, $35          'Instrument 0
        byte    $02, $EF, $AD, $79, $35          'Instrument 1
        byte    $FF                              'End of Instrument list
        byte    $63, $00, $83, $15, $63
        byte    $5C, $20, $93, $15, $24
        byte    $1E, $40, $9C, $15, $49
        byte    $40, $60, $83, $15, $52
        byte    $45, $20, $F7, $0F, $0D
        byte    $09, $41, $06, $0F, $0D
        byte    $09, $20, $F7, $0F, $0D
        byte    $09, $41, $06, $0F, $0D
        byte    $09, $20, $F7, $0F, $0D
        byte    $09, $41, $06, $0F, $0D
        byte    $09, $80, $F7, $0F, $0D
        byte    $09, $A1, $06, $0F, $0D 
        byte    $09, $60, $F7, $0F, $0D
        byte    $09, $21, $06, $0F, $0D
        byte    $09, $40, $F7, $0F, $0D
        byte    $09, $81, $06, $0F, $0D
        byte    $09, $A0, $F7, $0F, $0D
        byte    $09, $61, $06, $0F, $0D
        byte    $09, $20, $F7, $0F, $0D
        byte    $09, $41, $06, $0F, $0D
        byte    $09, $20, $F7, $0F, $0D 
        byte    $09, $81, $06, $0F, $80, $B2
        byte    $ff

_SONG_dk_hammer
        byte    $03, $F8, $48, $22, $11          'Instrument 0
        byte    $02, $EF, $AD, $79, $35          'Instrument 1
        byte    $FF                              'End of Instrument list
        byte    $00, $01, $5D, $0B, $0F
        byte    $10, $21, $5D, $0B, $07
        byte    $08, $41, $5D, $0B, $07
        byte    $08, $61, $5D, $0B, $0F
        byte    $10, $81, $5D, $0B, $0F
        byte    $10, $A1, $B8, $0B, $0F
        byte    $10, $01, $5D, $0B, $0F
        byte    $10, $21, $B8, $0B, $0F
        byte    $10, $41, $5D, $0B, $0F
        byte    $10, $61, $B8, $0B, $0F
        byte    $10, $81, $B8, $0B, $07
        byte    $08, $A1, $B8, $0B, $07
        byte    $08, $01, $B8, $0B, $0F
        byte    $10, $21, $B8, $0B, $0F
        byte    $10, $42, $0B, $0B, $0F
        byte    $10, $61, $B8, $0B, $0F
        byte    $10, $82, $0B, $0B, $0F
        byte    $10, $A1, $B8, $0B, $0F
        byte    $10, $00, $01, $01, $01
        byte    $ff

_SONG_dk_how_high
        byte    $03, $EF, $AD, $79, $35          'Instrument 0
        byte    $02, $EF, $AD, $79, $35          'Instrument 1
        byte    $FF                              'End of Instrument list
        byte    $00, $01, $06, $0F, $20
        byte    $25, $21, $4A, $0F, $0F
        byte    $18, $41, $88, $0F, $21
        byte    $23, $61, $4A, $0F, $0E
        byte    $17, $81, $26, $0F, $0D
        byte    $15, $A1, $4A, $0F, $0D
        byte    $15, $01, $06, $0F, $62
        byte    $18, $21, $06, $0F, $14
        byte    $12, $41, $26, $0F, $08
        byte    $07, $61, $4A, $0F, $06
        byte    $05, $81, $5D, $0F, $06
        byte    $04, $A1, $88, $0F, $08
        byte    $06, $21, $B8, $0F, $07
        byte    $06, $41, $EE, $0F, $09
        byte    $09, $62, $0B, $0F, $2D
        byte    $ff
                                
_SONG_end                                                                                                  
        byte    $ff 
' =/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/
' =/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/ HDMF LITE, (START)    /=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/
' =/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/
CON 
  HDMF_TIMEBASE_TICK     = 80_000_000 / 100         ' HDMF uses a 10 mesc tick (100 ticks per second).
                                                    ' HDMF_TICK is expressed in system clock ticks
                                                    '     so at 80MHz is 80,000,000 / 100
                                                       
  HDMF_DURATION_TICK     = snd#SAMPLE_RATE / 100    ' HDMF duration is also 100 ticks per second
                                                    ' DURATION_TICK is expressed in sound driver sample
                                                    '     rate ticks so is SAMPLE_RATE / 100
                                                       
  'Player states   
  HDMF__STATE_IDLE        = %00000000  
  HDMF__STATE_START_SONG  = %00000001
  HDMF__STATE_PLAY_FETCH  = %00000100
  HDMF__STATE_PLAY_WAIT   = %00000101
  HDMF__CHECK_IS_PLAYING  = %00000100

  'Player flags
  HDMF__FLAG_LAST_NOTE_5_BYTES = %00000001
  HDMF__FLAG_LOOP_SONG         = %00000010
  HDMF__FLAG_FIRST_NOTE        = %00000100  


  HDMF_MAX_INSTRUMENTS    = 8
  HDMF_BYTES_PER_NOTE_MAX = 6


OBJ

  snd    : "NS_sound_drv_052_22khz_16bit.spin"   'Sound driver

VAR

  long hdmf1_state
  long hdmf1_i
  long hdmf1_song_ptr
  long hdmf1_song_start_ptr
  byte hdmf1_flags
  long hdmf1_current_note_time
  long hdmf1_note_delta_time
  byte hdmf1_note_data[HDMF_BYTES_PER_NOTE_MAX]
  byte hdmf1_instrument_wave[HDMF_MAX_INSTRUMENTS]
  long hdmf1_instrument_envelope[HDMF_MAX_INSTRUMENTS]

  long hdmf1_P_freq                                  
  long hdmf1_P_duration
  long hdmf1_P_channel
  long hdmf1_P_volume
  long hdmf1_P_time
  long hdmf1_P_instrument
  
' =/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/
' =/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/ HDMF LITE, (END)      /=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/
' =/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/=/ 