'///////////////////////////////////////////////////////////////////////
' 
'' Hydra Dense Music Format (HDMF) Lite Driver 
'' AUTHOR: Eric Moyer
'' LAST MODIFIED: 05.26.07
'' VERSION 1.0
''
'' For the latest version search the parallax forums at:
''    http://forums.parallax.com 
''                                                 
''
'' //////////////////////////////////////////////////////////////////////
''
''  Revision History
''
''  Rev  Date      Description
''  ---  --------  ---------------------------------------------------
''  001  05-22-07  Initial Creation                  
''
'' //////////////////////////////////////////////////////////////////////  
''
''  Open issues:
''     Known bug in cnt math can cause music playback to stop when cnt wraps.
''
''  Documentation:
''  ==============
''  Please refer to the 'HDMF User's Manual' for extensive documentation on this
''  driver, the creation of HDMF assets (i.e. music data), and operation of the
''  'HDMF Converter' Windows application which can be used to covert MIDI data into
''  HDMF assets.      
''        
''  NOTE: This is a single cog driver designed for use in games with simple music
''        requirements where only a single playback cog is available.  If  more
''        than 1 cog is available please use the standard HDMF Driver which has
''        far superior playback capabilities.
''
'' //////////////////////////////////////////////////////////////////////  

'///////////////////////////////////////////////////////////////////////
' CONSTANTS SECTION ////////////////////////////////////////////////////
'///////////////////////////////////////////////////////////////////////

CON
  
  HDMF_TIMEBASE_TICK     = 80_000_000 / 100         ' HDMF uses a 10 mesc tick (100 ticks per second).
                                                    ' HDMF_TICK is expressed in system clock ticks
                                                    '     so at 80MHz is 80,000,000 / 100
                                                       
  HDMF_DURATION_TICK     = snd#SAMPLE_RATE / 100    ' HDMF duration is also 100 ticks per second
                                                    ' DURATION_TICK is expressed in sound driver sample
                                                    '     rate ticks so is SAMPLE_RATE / 100
                                                       
  'Player states   
  HDMF__STATE_IDLE        = %00000000  
  HDMF__STATE_START_SONG  = %00000001
  HDMF__STATE_PLAY_FETCH  = %00000100
  HDMF__STATE_PLAY_WAIT   = %00000101
  HDMF__CHECK_IS_PLAYING  = %00000100

  'Player flags
  HDMF__FLAG_LAST_NOTE_5_BYTES = %10000000  'Internal flag
  HDMF__FLAG_FIRST_NOTE        = %01000000  'Internal flag
  HDMF__FLAG_LOOP_SONG         = %00000001  'User flag
  
  HDMF__USER_FLAG_MASK         = %00000001  


  HDMF_MAX_INSTRUMENTS    = 8
  HDMF_BYTES_PER_NOTE_MAX = 6


OBJ

  snd    : "NS_sound_drv_052_22khz_16bit.spin"   'Sound driver

VAR

  long hdmf1_state
  long hdmf1_i
  long hdmf1_song_ptr
  long hdmf1_song_start_ptr
  long hdmf1_note_start_ptr
  byte hdmf1_flags
  long hdmf1_current_note_time
  long hdmf1_note_delta_time
  byte hdmf1_note_data[HDMF_BYTES_PER_NOTE_MAX]
  byte hdmf1_instrument_wave[HDMF_MAX_INSTRUMENTS]
  long hdmf1_instrument_envelope[HDMF_MAX_INSTRUMENTS]

  long hdmf1_P_freq                                  
  long hdmf1_P_duration
  long hdmf1_P_channel
  long hdmf1_P_volume
  long hdmf1_P_time
  long hdmf1_P_instrument

  long debug_data_ptr
  long note_number 
  long debug_data[7]

PUB start(arg_debug_data_ptr)
'' Starts the driver
''
  snd.start(7)
  hdmf1_state    := HDMF__STATE_IDLE
  debug_data_ptr := arg_debug_data_ptr
  DIRA[0]        := 1 'Enable debug LED as output

  'Play startup ping
  snd.PlaySoundFM(0, snd#SHAPE_SINE, 600, CONSTANT( snd#SAMPLE_RATE * 2 ), 255, $3579_ADEF )
  
PUB stop
'' Stops the driver. Frees all used cogs.
''
  snd.stop
  
PUB play_song(song_address, user_flags)
'' Starts song playback.
''
''   song_address:    The starting address of the song data (in RAM).
''                    EEPROM playback is not supported in the Lite deiver.  If EEPROM
''                    playback is needed use the standard HDMF driver.
''
''   user_flags:      Playbak flags.  Multiple flags can be OR'd together.
''                    Supported flags:
''                       HDMF__FLAG_LOOP_SONG   Loop the song forever. ( Use stop_song() to stop it )
''
  hdmf1_state := HDMF__STATE_START_SONG
  hdmf1_song_start_ptr := song_address
  hdmf1_flags := (user_flags & HDMF__USER_FLAG_MASK) 'Set new user flags
          
PUB stop_song
'' Stops a song in progress.                   
''
  hdmf1_state := HDMF__STATE_IDLE
  
PUB song_is_playing
'' Returns 'true' if a song is currently playing.
'' Returns 'false' otherwise.
''
  if hdmf1_state & HDMF__CHECK_IS_PLAYING
    return (true)
  else
    return (false)
    
PUB do_playback 
'' Processes music playack.
''
'' This function should be called AT LEAST once in your main game loop.  For improved
'' playback performace call do_playback multiple times (as evenly distributed in time
'' as possible) throughout your main code loop.
''
'' NOTE: If you dont call do_playback() you'll never hear anything.
''
    
  'START SONG >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
  if hdmf1_state == HDMF__STATE_START_SONG
   
    'Init flags
    hdmf1_flags &= !(HDMF__FLAG_LAST_NOTE_5_BYTES)
    hdmf1_flags |= HDMF__FLAG_FIRST_NOTE
    note_number := 0
    
    'Read instruments
    hdmf1_song_ptr := hdmf1_song_start_ptr
    hdmf1_i := 0
    repeat while (hdmf1_i =< HDMF_MAX_INSTRUMENTS - 1) and (byte[hdmf1_song_ptr] <> $ff) 
      hdmf1_instrument_wave[hdmf1_i] := byte[hdmf1_song_ptr]
      ++hdmf1_song_ptr
      hdmf1_instrument_envelope[hdmf1_i] := byte[hdmf1_song_ptr] | byte[hdmf1_song_ptr+1] << 8 | byte[hdmf1_song_ptr+2] << 16 | byte[hdmf1_song_ptr+3] << 24
      hdmf1_song_ptr += 4
      'Increment instrument index
      ++hdmf1_i      
      
    'skip over 'end of instruments' 0xff
    ++hdmf1_song_ptr
    
    hdmf1_state := HDMF__STATE_PLAY_FETCH
    hdmf1_note_start_ptr := hdmf1_song_ptr
   
  'FETCH NOTE >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> 
  elseif hdmf1_state == HDMF__STATE_PLAY_FETCH
    'Fetch song data from RAM
    if hdmf1_flags & HDMF__FLAG_LAST_NOTE_5_BYTES
      'One byte of note data already in buffer.  Read 5 more.
      repeat hdmf1_i from 1 to 5
        hdmf1_note_data[hdmf1_i] := byte[hdmf1_song_ptr]
        hdmf1_song_ptr++
    else
      'Read 6 new bytes of note data
      repeat hdmf1_i from 0 to 5
        hdmf1_note_data[hdmf1_i] := byte[hdmf1_song_ptr]  
        hdmf1_song_ptr++

    if (debug_data_ptr) and (note_number == 0)
      long[debug_data_ptr+constant(7*4)] := hdmf1_song_ptr
      
    hdmf1_note_delta_time := hdmf1_note_data[0] * HDMF_TIMEBASE_TICK  
   
    'Init the note timebase on the first note played
    if hdmf1_flags & HDMF__FLAG_FIRST_NOTE
      hdmf1_current_note_time := cnt
      hdmf1_flags &= !HDMF__FLAG_FIRST_NOTE
     
    'If not end of song
    if (hdmf1_note_data[0] <> $ff)
      
      hdmf1_P_time := hdmf1_current_note_time + hdmf1_note_delta_time
      hdmf1_P_channel := (hdmf1_note_data[1] & $e0) >> 5
      hdmf1_P_freq := (((hdmf1_note_data[1] & $1f) << 8) | hdmf1_note_data[2] ) << 1
      hdmf1_P_volume := hdmf1_note_data[3] << 3
      hdmf1_P_instrument := (hdmf1_note_data[3] & $e0) >> 5
   
      if hdmf1_note_data[4] & $80
        hdmf1_P_duration := ((hdmf1_note_data[4] & $7f) << 8 | hdmf1_note_data[5]) * HDMF_DURATION_TICK
        hdmf1_flags &= !HDMF__FLAG_LAST_NOTE_5_BYTES
      else
        hdmf1_P_duration := hdmf1_note_data[4] * HDMF_DURATION_TICK
        'Note was only 5 bytes long, so save the 6th byte for the next note
        hdmf1_note_data[0] := hdmf1_note_data[5]
        hdmf1_flags |= HDMF__FLAG_LAST_NOTE_5_BYTES
   
      'Enter the wait state (note will play when its scheduled time arrives)
      hdmf1_state := HDMF__STATE_PLAY_WAIT
    else
      'Song over.
      if hdmf1_flags & HDMF__FLAG_LOOP_SONG
        'Looping.  Restart song from beginning.
        hdmf1_song_ptr := hdmf1_note_start_ptr
        hdmf1_state := HDMF__STATE_PLAY_FETCH
        hdmf1_flags &= !HDMF__FLAG_LAST_NOTE_5_BYTES
        hdmf1_flags |= HDMF__FLAG_FIRST_NOTE   
      else
        'Done. Enter idle state.
        hdmf1_state := HDMF__STATE_IDLE
   
  'WAIT NOTE >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
  elseif hdmf1_state == HDMF__STATE_PLAY_WAIT    
    'If not waiting to cross end of counter wrap
    if not (((hdmf1_current_note_time>>1) > $40000000) and ((hdmf1_P_time>>1) < $40000000) and ((cnt>>1) > $40000000))
      OUTA [0] := !INA[0] 'Toggle debug LED 
      'If note's scheduled play time is reached
      if ((cnt>>1) => (hdmf1_P_time>>1))
        snd.PlaySoundFM(hdmf1_P_channel, hdmf1_instrument_wave[hdmf1_P_instrument], hdmf1_P_freq, hdmf1_P_duration, hdmf1_P_volume, hdmf1_instrument_envelope[hdmf1_P_instrument] )
        hdmf1_current_note_time := hdmf1_P_time
        'Enter the fetch state to fetch the next note
        hdmf1_state := HDMF__STATE_PLAY_FETCH
   
        if (debug_data_ptr) and (note_number == 0)
          'Debug enabled.  Dump debug data.
          long[debug_data_ptr+constant(0*4)] := hdmf1_note_data[0] << 24 | hdmf1_note_data[1] << 16 | hdmf1_note_data[2] << 8 | hdmf1_note_data[3]
          long[debug_data_ptr+constant(1*4)] := hdmf1_note_data[4] << 24 | hdmf1_note_data[5] << 16
          long[debug_data_ptr+constant(2*4)] := hdmf1_P_time
          long[debug_data_ptr+constant(3*4)] := hdmf1_P_channel
          long[debug_data_ptr+constant(4*4)] := hdmf1_P_freq
          long[debug_data_ptr+constant(5*4)] := hdmf1_P_volume
          long[debug_data_ptr+constant(6*4)] := hdmf1_P_duration

        ++note_number
  